/*=============================================================================
vitalnix/C-spark - User Management UI
data.c - Data processing
  Copyright (C) Jan Engelhardt <jengelh [at] linux01 gwdg de>, 1999 - 2003
  -- License restrictions apply (Dual FAL/GPL2).
  -- For details see doc/FAL.txt and doc/GPL2.txt.
=============================================================================*/
#include <ctype.h>
#include <stdio.h>
#include <string.h>

#include <libHX.h>
#include "cui/global.h"

static char *transform7(byte *, char *, size_t);

static const struct stab {
    char *set, *out;
} subst_tab[] = {
    {"", "ae"}, {"", "oe"}, {"", "ue"}, {"", "ss"},
    {"", "a"}, {"", "c"}, {"", "e"}, {"", "i"},
    {"", "o"}, {"", "u"}, {"", "l"}, {"", "m"},
    {"", "n"}, {"", "r"}, {"", "y"}, {NULL, NULL},
};

//-----------------------------------------------------------------------------
char *fixup_login_name(char *ln, unsigned int idx, char *dest, size_t dsize) {
    /* Login names are fixup'ed by adding a unique number them, necessarily
    truncating part of the original login name. */
    char idxstr[16], tln[16];
    snprintf(idxstr, 16, "%u", idx);
    snprintf(tln, 8 - strlen(idxstr) + 1, "%s", ln);
    snprintf(dest, dsize, "%s%s", tln, idxstr);
    return dest;
}

char *make_login_name(char *in_nname, char *in_vname) {
    /* This takes the name (in_nname) and prenom (in_vname) and makes a 7-bit
    clean login name in the style VNNNNNN. Numerical indexes are done in
    fixup_login_name() (called from the main code). */
    char buf[8], fix_nname[8], fix_vname[8], *p;

    memset(fix_nname, '\0', 8);
    memset(fix_vname, '\0', 8);
    transform7(in_vname, fix_vname, 8);

    if(in_nname != NULL) {
        if((p = strchr(in_nname, ' ')) != NULL) { in_nname = ++p; }
        transform7(in_nname, fix_nname, 8);
        snprintf(buf, 8, "%c%s", *fix_vname, fix_nname);
    } else {
        snprintf(buf, 8, "%s", fix_vname);
    }

    return HX_strdup(HX_strlower(buf));
}

int only_digits(char *p) {
    while(*p) {
        if(!isdigit(*p++)) { return 0; }
    }
    return 1;
}

//-----------------------------------------------------------------------------
static char *transform7(byte *src, char *dest, size_t dsize) {
    /* Transform a name (possibly with umlauts, etc.) into a 7-bit clean,
    A-to-Z-only login name. */
    dest[--dsize] = '\0';
    while(dsize && *src != '\0') {
        if((*src >= 'A' && *src <= 'Z') || (*src >= 'a' && *src <= 'z')) {
            *dest++ = *src;
            --dsize;
        } else if(*src >= 128) {
            // This loop is still faster than a regex
            const struct stab *sp = subst_tab;
            while(sp->set != NULL) {
                if(strchr(sp->set, *src) != NULL) {
                    size_t l = strlen(sp->out);
                    if(dsize >= l) {
                        strncpy(dest, sp->out, l);
                        dest += l;
                        dsize -= l;
                    } else {
                        /* Do not let "abcdea" through, because if there was
                        too few space, the "" gets skipped but dsize is still
                        big enough to let the "a" in. */
                        dsize = 0;
                    }
                    break;
                }
                ++sp;
            }
        }
        ++src;
    }
    return dest;
}

//==[ End of file ]============================================================
